Attribute VB_Name = "ligneligne"


'---------------------------------------------------------------------------
Function DistLigneLigne(line0 As Line3, line1 As Line3, s As Double, t As Double) As Double

    Dim diff As Point3
    Dim A  As Double
    Dim B As Double
    Dim C As Double
    Dim D As Double
    Dim E As Double   ' -Dot(line1.Direction,diff), defer until needed
    Dim F As Double
    Dim det As Double
    Dim invDet As Double
    
     diff = VecSub(line0.origin, line1.origin)
     A = Dot(line0.Direction, line0.Direction)
     B = -Dot(line0.Direction, line1.Direction)
     C = Dot(line1.Direction, line1.Direction)
     D = Dot(line0.Direction, diff)
     'E  ' -Dot(line1.Direction,diff), defer until needed
     F = Dot(diff, diff)
     det = Abs(A * C - B * B) ' A*C-B*B = |Cross(M0,M1)|^2 >= 0


    If (det >= EPSILON) Then
        ' lines are not parallel
        E = -Dot(line1.Direction, diff)
        invDet = 1 / det
        s = (B * E - C * D) * invDet
        t = (B * D - A * E) * invDet
        DistLigneLigne = Sqr(Abs((s * (A * s + B * t + 2 * D) + t * (B * s + C * t + 2 * E) + F)))
 
    Else
        ' lines are parallel, select any closest pair of points
        s = -D / A
        t = 0
        DistLigneLigne = Sqr(Abs((D * s + F)))
    End If
End Function
'---------------------------------------------------------------------------
Function DistLigneRayon(line As Line3, ray As Line3, s As Double, t As Double) As Double


    Dim diff As Point3
    Dim A As Double
    Dim B As Double
    Dim C As Double
    Dim D As Double
    Dim E  As Double
    Dim F As Double
    Dim det As Double
    Dim invDet As Double
    
     diff = VecSub(line.origin, ray.origin)
     
     A = Dot(line.Direction, line.Direction)
     B = -Dot(line.Direction, ray.Direction)
     C = Dot(ray.Direction, ray.Direction)
     D = Dot(line.Direction, diff)
     'E  ' -Dot(ray.Direction,diff), defer until needed
     F = Dot(diff, diff)
     det = Abs(A * C - B * B) ' A*C-B*B = |Cross(M0,M1)|^2 >= 0

    If (det >= tolerance) Then
        E = -Dot(ray.Direction, diff)
        t = B * D - A * E

        If (t >= 0) Then
            ' two interior points are closest, one on line and one on ray
            invDet = 1 / det
            s = (B * E - C * D) * invDet
            t = t * invDet
            DistLigneRayon = Sqr(Abs((s * (A * s + B * t + 2 * D) + t * (B * s + C * t + 2 * E) + F)))
        Else
            ' end point of ray and interior point of line are closest
            s = -D / A
            t = 0
            DistLigneRayon = Sqr(Abs((D * s + F)))
        End If
    Else
        ' lines are parallel, closest pair with one point at ray origin
        s = -D / A
        t = 0
        DistLigneRayon = Sqr(Abs((D * s + F)))
    End If
End Function
'---------------------------------------------------------------------------
Function DistLigneSegment(line As Line3, seg As Line3, s As Double, t As Double) As Double

    Dim diff As Point3
    Dim A As Double
    Dim B As Double
    Dim C As Double
    Dim D As Double
    Dim E  As Double
    Dim F As Double
    Dim det As Double
    Dim invDet As Double
    Dim tmp As Double
    
     diff = VecSub(line.origin, seg.origin)
     A = Dot(line.Direction, line.Direction)
     B = -Dot(line.Direction, seg.Direction)
     C = Dot(seg.Direction, seg.Direction)
     D = Dot(line.Direction, diff)
     'E  ' -Dot(seg.Direction,diff), defer until needed
     F = Dot(diff, diff)
     det = Abs(A * C - B * B) ' A*C-B*B = |Cross(M0,M1)|^2 >= 0

    If (det >= EPSILON) Then
        E = -Dot(seg.Direction, diff)
        t = B * D - A * E

        If (t >= 0) Then
            If (t <= det) Then
                ' two interior points are closest, one on line and one on
                ' line segment
                 invDet = 1 / det
                s = (B * E - C * D) * invDet
                t = t * invDet
                DistLigneSegment = Sqr(Abs((s * (A * s + B * t + 2 * D) + t * (B * s + C * t + 2 * E) + F)))
                ' ' exit function
            Else
                ' end point e1 of line segment and interior point of line
                ' are closest
                 tmp = B + D
                s = -tmp / A
                t = 1
                DistLigneSegment = Sqr(Abs((tmp * s + C + 2 * E + F)))
                ' ' exit function
            End If
        Else
            ' end point e0 of line segment and interior point of line
            ' are closest
            s = -D / A
            t = 0
            DistLigneSegment = Sqr(Abs((D * s + F)))
            ' exit function
        End If
    Else
        ' lines are parallel, closest pair with one point at segment origin
        s = -D / A
        t = 0
        DistLigneSegment = Sqr(Abs((D * s + F)))
        ' exit function
    End If
End Function

'---------------------------------------------------------------------------
Function DistRayonRayon(ray0 As Line3, ray1 As Line3, s As Double, t As Double) As Double

    Dim diff As Point3
    Dim A As Double
    Dim B As Double
    Dim C As Double
    Dim D As Double
    Dim E  As Double
    Dim F As Double
    Dim det As Double
    Dim invDet As Double
    Dim tmp As Double
    
     diff = VecSub(ray0.origin, ray1.origin)
     A = Dot(ray0.Direction, ray0.Direction)
     B = -Dot(ray0.Direction, ray1.Direction)
     C = Dot(ray1.Direction, ray1.Direction)
     D = Dot(ray0.Direction, diff)
    ' E  ' -Dot(ray1.Direction,diff), defer until needed
     F = Dot(diff, diff)
     det = Abs(A * C - B * B) ' A*C-B*B = |Cross(M0,M1)|^2 >= 0

    If (det >= EPSILON) Then

        ' rays are not parallel
        E = -Dot(ray1.Direction, diff)
        s = B * E - C * D
        t = B * D - A * E

        If (s >= 0) Then
            If (t >= 0) Then    ' region 0 (interior)
                ' minimum at two interior points of rays
                 invDet = 1 / det
                s = s * invDet
                t = t * invDet
                DistRayonRayon = Sqr(Abs((s * (A * s + B * t + 2 * D) + t * (B * s + C * t + 2 * E) + F)))
                ' exit function
            Else  ' region 3 (side)
                t = 0
                If (D >= 0) Then
                    s = 0
                    DistRayonRayon = Sqr(Abs((F)))
                    ' exit function
                Else
                    s = -D / A
                    DistRayonRayon = Sqr(Abs((D * s + F)))
                    ' exit function
                 End If
            End If
       
        Else

            If (t >= 0) Then   ' region 1 (side)
                s = 0
                If (E >= 0) Then
                    t = 0
                    DistRayonRayon = Sqr(Abs((F)))
                    ' exit function
                Else
                    t = -E / C
                    DistRayonRayon = Sqr(Abs((E * t + F)))
                    ' exit function
                End If
            Else  ' region 2 (corner)
                If (D < 0) Then
                    s = -D / A
                    t = 0
                    DistRayonRayon = Sqr(Abs((D * s + F)))
                    ' exit function
                Else
                    s = 0
                    If (E >= 0) Then
                        t = 0
                        DistRayonRayon = Sqr(Abs((F)))
                        ' exit function
                    Else
                        t = -E / C
                        DistRayonRayon = Sqr(Abs((E * t + F)))
                        ' exit function
                    End If
                End If
            End If
        End If
    Else
        ' rays are parallel
        If (B > 0) Then
            ' opposite direction vectors
            t = 0
            If (D >= 0) Then
                s = 0
                DistRayonRayon = Sqr(Abs((F)))
                ' exit function
            Else
                s = -D / A
                DistRayonRayon = Sqr(Abs((D * s + F)))
            End If
        Else
            ' same direction vectors
            If (D >= 0) Then
                E = -Dot(ray1.Direction, diff)
                s = 0
                t = -E / C
                DistRayonRayon = Sqr(Abs((E * t + F)))
                ' exit function
            Else
                s = -D / A
                t = 0
                DistRayonRayon = Sqr(Abs((D * s + F)))
                ' exit function
            End If
        End If
    End If
End Function
'---------------------------------------------------------------------------
Function DistRayonSegment(ray As Line3, seg As Line3, s As Double, t As Double) As Double

    Dim diff As Point3
    Dim A As Double
    Dim B As Double
    Dim C As Double
    Dim D As Double
    Dim E  As Double
    Dim F As Double
    Dim det As Double
    Dim invDet As Double
    Dim tmp As Double
    
     diff = VecSub(ray.origin, seg.origin)
     A = Dot(ray.Direction, ray.Direction)
     B = -Dot(ray.Direction, seg.Direction)
     C = Dot(seg.Direction, seg.Direction)
     D = Dot(ray.Direction, diff)
    ' E  ' -Dot(seg.Direction,diff), defer until needed
     F = Dot(diff, diff)
     det = Abs(A * C - B * B) ' A*C-B*B = |Cross(M0,M1)|^2 >= 0


    If (det >= EPSILON) Then
        ' ray and segment are not parallel
        E = -Dot(seg.Direction, diff)
        s = B * E - C * D
        t = B * D - A * E

        If (s >= 0) Then
            If (t >= 0) Then
                If (t <= det) Then   ' region 0
                    ' minimum at interior points of ray and segment
                    invDet = 1 / det
                    s = s * invDet
                    t = t * invDet
                    DistRayonSegment = Sqr(Abs((s * (A * s + B * t + 2 * D) + t * (B * s + C * t + 2 * E) + F)))
                    ' exit function
                Else  ' region 1
                    t = 1
                    If (D >= 0) Then
                        s = 0
                        DistRayonSegment = Sqr(Abs((C + 2 * E + F)))
                        ' exit function
                    Else
                        s = -D / A
                        DistRayonSegment = Sqr(Abs(((D + 2 * B) * s + C + 2 * E + F)))
                        ' exit function
                    End If
                End If
            Else  ' region 5
                t = 0
                If (D >= 0) Then
                    s = 0
                    DistRayonSegment = Sqr(Abs((F)))
                    ' exit function
                Else
                    s = -D / A
                    DistRayonSegment = Sqr(Abs((D * s + F)))
                    ' exit function
                End If
            End If
        Else
            If (t <= 0) Then   ' region 4
                If (D < 0) Then
                    s = -D / A
                    t = 0
                    DistRayonSegment = Sqr(Abs((D * s + F)))
                    ' exit function
                Else
                    s = 0
                    If (E >= 0) Then
                        t = 0
                        DistRayonSegment = Sqr(Abs((F)))
                        ' exit function
                    ElseIf (-E >= C) Then
                        t = 1
                        DistRayonSegment = Sqr(Abs((C + 2 * E + F)))
                        ' exit function

                    Else

                        t = -E / C
                        DistRayonSegment = Sqr(Abs((E * t + F)))
                        ' exit function
                    End If
                End If
            ElseIf (t <= det) Then   ' region 3
                s = 0
                If (E >= 0) Then
                    t = DistRayonSegment = Sqr(Abs((F)))
                    ' exit function
                ElseIf (-E >= C) Then

                    t = 1
                    DistRayonSegment = Sqr(Abs((C + 2 * E + F)))
                    ' exit function
                Else

                    t = -E / C
                    DistRayonSegment = Sqr(Abs((E * t + F)))
                    ' exit function
                End If

            Else  ' region 2

                tmp = B + D
                If (tmp < 0) Then

                    s = -tmp / A
                    t = 1
                    DistRayonSegment = Sqr(Abs((tmp * s + C + 2 * E + F)))
                    ' exit function

                Else

                    s = 0
                    If (E >= 0) Then

                        t = 0
                        DistRayonSegment = Sqr(Abs((F)))
                        ' exit function

                    ElseIf (-E >= C) Then

                        t = 1
                        DistRayonSegment = Sqr(Abs((C + 2 * E + F)))
                        ' exit function

                    Else

                        t = -E / C
                        DistRayonSegment = Sqr(Abs((E * t + F)))
                        ' exit function
                    End If
                End If
           End If
        End If
    Else
        ' ray and segment are parallel
        If (B > 0) Then
            ' opposite direction vectors
            t = 0
            If (D >= 0) Then
                s = 0
                DistRayonSegment = Sqr(Abs((F)))
                ' exit function
            Else
                s = -D / A
                DistRayonSegment = Sqr(Abs((D * s + F)))
                ' exit function
            End If


            ' same direction vectors
            E = -Dot(seg.Direction, diff)
            t = 1
            tmp = B + D
            If (tmp >= 0) Then

                s = 0
                DistRayonSegment = Sqr(Abs((C + 2 * E + F)))
                ' exit function

            Else

                s = -tmp / A
                DistRayonSegment = Sqr(Abs((tmp * s + C + 2 * E + F)))
                ' exit function
            End If
        End If
    End If
End Function
'---------------------------------------------------------------------------
Function DistSegmentSegment(seg0 As Line3, seg1 As Line3, s As Double, t As Double)


    Dim diff As Point3
    Dim A As Double
    Dim B As Double
    Dim C As Double
    Dim D As Double
    Dim E  As Double
    Dim F As Double
    Dim det As Double
    Dim invDet As Double
    Dim tmp As Double
    
    
     diff = VecSub(seg0.origin, seg1.origin)
     A = Dot(seg0.Direction, seg0.Direction)
     B = -Dot(seg0.Direction, seg1.Direction)
     C = Dot(seg1.Direction, seg1.Direction)
     D = Dot(seg0.Direction, diff)
    ' E  ' -Dot(seg1.Direction,diff), defer until needed
     F = Dot(diff, diff)
     det = Abs(A * C - B * B) ' A*C-B*B = |Cross(M0,M1)|^2 >= 0


    If (det >= EPSILON) Then
        ' line segments are not parallel
        E = -Dot(seg1.Direction, diff)
        s = B * E - C * D
        t = B * D - A * E
        
        If (s >= 0) Then
            If (s <= det) Then
                If (t >= 0) Then
                    If (t <= det) Then   ' region 0 (interior)
                        ' minimum at two interior points of 3D lines
                        invDet = 1 / det
                        s = s * invDet
                        t = t * invDet
                        DistSegmentSegment = Sqr(Abs((s * (A * s + B * t + 2 * D) + t * (B * s + C * t + 2 * E) + F)))
                        ' exit function
                    Else  ' region 3 (side)
                        t = 1
                        tmp = B + D
                        If (tmp >= 0) Then
                            s = 0
                            DistSegmentSegment = Sqr(Abs((C + 2 * E + F)))
                            ' exit function
                        ElseIf (-tmp >= A) Then
                        
                            s = 1
                            DistSegmentSegment = Sqr(Abs((A + C + F + 2 * (E + tmp))))
                            ' exit function
                        
                        Else
                        
                            s = -tmp / A
                            DistSegmentSegment = Sqr(Abs((tmp * s + C + 2 * E + F)))
                            ' exit function
                        End If
                    End If
                Else  ' region 7 (side)

                    t = 0
                    If (D >= 0) Then
                        s = 0
                        DistSegmentSegment = Sqr(Abs((F)))
                        ' exit function
                    ElseIf (-D >= A) Then
                        s = 1
                        DistSegmentSegment = Sqr(Abs((A + 2 * D + F)))
                        ' exit function
                    Else
                        s = -D / A
                        DistSegmentSegment = Sqr(Abs((D * s + F)))
                        ' exit function
                    End If
                End If
            Else
                If (t >= 0) Then
                    If (t <= det) Then    ' region 1 (side)
                        s = 1
                        tmp = B + E
                        If (tmp >= 0) Then
                            t = 0
                            DistSegmentSegment = Sqr(Abs((A + 2 * D + F)))
                            ' exit function

                        ElseIf (-tmp >= C) Then

                            t = 1
                            DistSegmentSegment = Sqr(Abs((A + C + F + 2 * (D + tmp))))
                            ' exit function
                        Else
                            t = -tmp / C
                            DistSegmentSegment = Sqr(Abs((tmp * t + A + 2 * D + F)))
                            ' exit function
                        End If
                    
                    Else  ' region 2 (corner)

                        tmp = B + D
                        If (-tmp <= A) Then
                            t = 1
                            If (tmp >= 0) Then
                                s = 0
                                DistSegmentSegment = Sqr(Abs((C + 2 * E + F)))
                                ' exit function
                            Else
                                 s = -tmp / A
                                 DistSegmentSegment = Sqr(Abs((tmp * s + C + 2 * E + F)))
                                 ' exit function
                            End If

                        Else

                            s = 1
                            tmp = B + E
                            If (tmp >= 0) Then
                                t = 0
                                DistSegmentSegment = Sqr(Abs((A + 2 * D + F)))
                                ' exit function
                            ElseIf (-tmp >= C) Then

                                t = 1
                                DistSegmentSegment = Sqr(Abs((A + C + F + 2 * (D + tmp))))
                                ' exit function

                            Else

                                t = -tmp / C
                                DistSegmentSegment = Sqr(Abs((tmp * t + A + 2 * D + F)))
                                ' exit function
                            End If
                        End If
                    End If
                Else  ' region 8 (corner)

                    If (-D < A) Then
                        t = 0
                        If (D >= 0) Then
                            s = 0
                            DistSegmentSegment = Sqr(Abs((F)))
                            ' exit function
                        Else
                            s = -D / A
                            DistSegmentSegment = Sqr(Abs((D * s + F)))
                            ' exit function
                        End If
                    Else
                        s = 1
                        tmp = B + E
                        If (tmp >= 0) Then
                            t = 0
                            DistSegmentSegment = Sqr(Abs((A + 2 * D + F)))
                            ' exit function

                        ElseIf (-tmp >= C) Then
                            t = 1
                            DistSegmentSegment = Sqr(Abs((A + C + F + 2 * (D + tmp))))
                            ' exit function
                        Else
                            t = -tmp / C
                            DistSegmentSegment = Sqr(Abs((tmp * t + A + 2 * D + F)))
                            ' exit function
                        End If
                    End If
                End If
            End If
        Else
            If (t >= 0) Then
                If (t <= det) Then   ' region 5 (side)
                    s = 0
                    If (E >= 0) Then
                        t = 0
                        DistSegmentSegment = Sqr(Abs((F)))
                        ' exit function
                    ElseIf (-E >= C) Then
                        t = 1
                        DistSegmentSegment = Sqr(Abs((C + 2 * E + F)))
                        ' exit function
                    Else
                        t = -E / C
                        DistSegmentSegment = Sqr(Abs((E * t + F)))
                        ' exit function
                    End If
                Else  ' region 4 (corner)
                    tmp = B + D
                    If (tmp < 0) Then
                        t = 1
                        If (-tmp >= A) Then
                            s = 1
                            DistSegmentSegment = Sqr(Abs((A + C + F + 2 * (E + tmp))))
                            ' exit function
                        Else
                            s = -tmp / A
                            DistSegmentSegment = Sqr(Abs((tmp * s + C + 2 * E + F)))
                            ' exit function
                        End If
                    Else
                        s = 0
                        If (E >= 0) Then
                            t = 0
                            DistSegmentSegment = Sqr(Abs((F)))
                            ' exit function
                        ElseIf (-E >= C) Then
                            t = 1
                            DistSegmentSegment = Sqr(Abs((C + 2 * E + F)))
                            ' exit function
                        
                        Else

                            t = -E / C
                            DistSegmentSegment = Sqr(Abs((E * t + F)))
                            ' exit function
                        End If
                    End If
                End If
            Else   ' region 6 (corner)
                If (D < 0) Then
                    t = 0
                    If (-D >= A) Then
                        s = 1
                        DistSegmentSegment = Sqr(Abs((A + 2 * D + F)))
                        ' exit function
                    Else
                        s = -D / A
                        DistSegmentSegment = Sqr(Abs((D * s + F)))
                        ' exit function
                   End If
                Else
                    s = 0
                    If (E >= 0) Then
                        t = 0
                        DistSegmentSegment = Sqr(Abs((F)))
                        ' exit function
                    ElseIf (-E >= C) Then
                        t = 1
                        DistSegmentSegment = Sqr(Abs((C + 2 * E + F)))
                        ' exit function
                    Else
                        t = -E / C
                        DistSegmentSegment = Sqr(Abs((E * t + F)))
                        ' exit function
                   End If
               End If
            End If
        End If
    Else
        ' line segments are parallel
        If (B > 0) Then
            ' direction vectors form an obtuse angle
            If (D >= 0) Then
                s = 0
                t = 0
                DistSegmentSegment = Sqr(Abs((F)))
                ' exit function
            ElseIf (-D <= A) Then
                s = -D / A
                t = 0
                DistSegmentSegment = Sqr(Abs((D * s + F)))
                ' exit function
            Else
                E = -Dot(seg1.Direction, diff)
                s = 1
                tmp = A + D
                If (-tmp >= B) Then
                    t = 1
                    DistSegmentSegment = Sqr(Abs((A + C + F + 2 * (B + D + E))))
                    ' exit function
                Else
                    t = -tmp / B
                    DistSegmentSegment = Sqr(Abs((A + 2 * D + F + t * (C * t + 2 * (B + E)))))
                    ' exit function
                End If
           End If
        Else
            ' direction vectors form an acute angle
            If (-D >= A) Then
                s = 1
                t = 0
                DistSegmentSegment = Sqr(Abs((A + 2 * D + F)))
                ' exit function
            ElseIf (D <= 0) Then
                s = -D / A
                t = 0
                DistSegmentSegment = Sqr(Abs((D * s + F)))
                ' exit function
            Else
                E = -Dot(seg1.Direction, diff)
                s = 0
                If (D >= -B) Then
                    t = 1
                    DistSegmentSegment = Sqr(Abs((C + 2 * E + F)))
                    ' exit function
                Else
                    t = -D / B
                    DistSegmentSegment = Sqr(Abs((F + t * (2 * E + C * t))))
                    ' exit function
                End If
            End If
        End If
    End If
End Function
'---------------------------------------------------------------------------

